// Copyright (c) 2018, codegrue. All rights reserved. Use of this source code
// is governed by the MIT license that can be found in the LICENSE file.

import 'dart:math';
import 'dart:typed_data';

import 'package:card_settings/card_settings.dart';
import 'package:card_settings/helpers/platform_functions.dart';
import 'package:file_picker_cross/file_picker_cross.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_cupertino_settings/flutter_cupertino_settings.dart';
import 'package:flutter_material_pickers/flutter_material_pickers.dart';

/// This is the file picker field
class CardSettingsFilePicker extends FormField<Uint8List> {
  CardSettingsFilePicker({
    Key key,
    bool autovalidate: false,
    FormFieldSetter<Uint8List> onSaved,
    FormFieldValidator<Uint8List> validator,
    Uint8List initialValue,
    this.visible = true,
    this.label = 'Label',
    String unattachDialogTitle,
    this.unattachDialogCancel = 'Cancel',
    this.unattachDialogConfirm = 'Unattach',
    this.onChanged,
    this.contentAlign,
    this.icon,
    this.labelAlign,
    this.requiredIndicator,
    this.style,
    this.showMaterialonIOS,
    this.fileType,
    this.fileExtension,
  })  : unattachDialogTitle = unattachDialogTitle ?? 'Unattach ' + label + "?",
        super(
            key: key,
            initialValue: initialValue,
            onSaved: onSaved,
            validator: validator,
            autovalidate: autovalidate,
            builder: (FormFieldState<Uint8List> field) =>
                (field as _CardSettingsFilePickerState)._build(field.context));

  final ValueChanged<Uint8List> onChanged;

  final String label;

  final String unattachDialogTitle;

  final String unattachDialogConfirm;

  final String unattachDialogCancel;

  final TextAlign labelAlign;

  final TextAlign contentAlign;

  final Icon icon;

  final Widget requiredIndicator;

  final bool visible;

  final TextStyle style;

  final bool showMaterialonIOS;

  final FileTypeCross fileType;

  final String fileExtension;

  @override
  _CardSettingsFilePickerState createState() => _CardSettingsFilePickerState();

  static String formatBytes(int bytes, int decimals) {
    if (bytes <= 0) return "0 B";
    const suffixes = ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"];
    var i = (log(bytes) / log(1024)).floor();
    return ((bytes / pow(1024, i)).toStringAsFixed(decimals)) +
        ' ' +
        suffixes[i];
  }
}

class _CardSettingsFilePickerState extends FormFieldState<Uint8List> {
  @override
  CardSettingsFilePicker get widget => super.widget as CardSettingsFilePicker;

  Widget _build(BuildContext context) {
    String formattedValue = (value == null)
        ? ''
        : CardSettingsFilePicker.formatBytes(value.length, 2);

    if (showCupertino(context, widget.showMaterialonIOS))
      return _buildCupertinoFilePicker(formattedValue);
    else
      return _buildMaterialFilePicker(formattedValue);
  }

  void onTap() {
    if (value == null) {
      showMaterialFilePicker(
        onChanged: (file) => setState(() {
          didChange(file);
          if (widget.onChanged != null) widget.onChanged(file);
        }),
      );
    } else {
      _showUnattachDialog();
    }
  }

  void _showUnattachDialog() async {
    var theme = Theme.of(context);
    var showIOS = (theme.platform == TargetPlatform.iOS ||
        theme.platform == TargetPlatform.macOS);

    if (!showIOS) {
      return showDialog<void>(
        context: context,
        builder: (BuildContext context) {
          return AlertDialog(
            title: Text(
              widget.unattachDialogTitle,
              style: Theme.of(context)
                  .textTheme
                  .headline6
                  .copyWith(color: Theme.of(context).textTheme.headline1.color),
            ),
            actions: <Widget>[
              FlatButton(
                child: Text(
                  widget.unattachDialogCancel,
                ),
                onPressed: () => Navigator.of(context).pop(),
              ),
              FlatButton(
                child: Text(widget.unattachDialogConfirm),
                onPressed: () {
                  didChange(null);
                  if (widget.onChanged != null) widget.onChanged(null);
                  Navigator.of(context).pop();
                },
              ),
            ],
          );
        },
      );
    } else {
      return showCupertinoDialog<void>(
        context: context,
        builder: (BuildContext context) {
          return CupertinoAlertDialog(
            title: Text(widget.unattachDialogTitle),
            actions: <Widget>[
              CupertinoDialogAction(
                child: Text(widget.unattachDialogCancel),
                onPressed: () => Navigator.of(context).pop(),
              ),
              CupertinoDialogAction(
                child: Text(widget.unattachDialogConfirm),
                onPressed: () {
                  didChange(null);
                  if (widget.onChanged != null) widget.onChanged(null);
                  Navigator.of(context).pop();
                },
              ),
            ],
          );
        },
      );
    }

    // return showPlatformDialog<void>(
    //   context: context,
    //   builder: (context) => PlatformAlertDialog(
    //     title: Text(
    //       widget.unattachDialogTitle,
    //       style: isMaterial(context)
    //           ? Theme.of(context)
    //               .textTheme
    //               .headline6
    //               .copyWith(color: Theme.of(context).textTheme.headline1.color)
    //           : null,
    //     ),
    //     actions: [
    //       PlatformDialogAction(
    //         child: PlatformText(widget.unattachDialogCancel),
    //         onPressed: () => Navigator.of(context).pop(),
    //       ),
    //       PlatformDialogAction(
    //         child: PlatformText(widget.unattachDialogConfirm),
    //         cupertino: (_, __) =>
    //             CupertinoDialogActionData(isDestructiveAction: true),
    //         onPressed: () {
    //           didChange(null);
    //           if (widget.onChanged != null) widget.onChanged(null);
    //           Navigator.of(context).pop();
    //         },
    //       )
    //     ],
    //   ),
    // );
  }

  Widget _buildCupertinoFilePicker(String formattedValue) {
    return Container(
      child: widget?.visible == false
          ? null
          : GestureDetector(
              onTap: onTap,
              child: CSControl(
                nameWidget: widget?.requiredIndicator != null
                    ? Text((widget?.label ?? "") + ' *')
                    : Text(widget?.label),
                contentWidget: Text(
                  formattedValue,
                  style: widget?.style ?? Theme.of(context).textTheme.subtitle1,
                  textAlign: widget?.contentAlign ??
                      CardSettings.of(context).contentAlign,
                ),
                style: CSWidgetStyle(icon: widget?.icon),
              ),
            ),
    );
  }

  Widget _buildMaterialFilePicker(String formattedValue) {
    return GestureDetector(
      onTap: onTap,
      child: CardSettingsField(
        label: widget?.label ?? "File",
        labelAlign: widget?.labelAlign,
        visible: widget?.visible ?? true,
        icon: widget?.icon ?? Icon(Icons.attach_file),
        requiredIndicator: widget?.requiredIndicator,
        errorText: errorText,
        content: _buildFieldContent(formattedValue),
        pickerIcon: value == null ? Icons.attach_file : Icons.clear,
      ),
    );
  }

  Widget _buildFieldContent(String formattedValue) {
    if (widget.fileType == FileTypeCross.image && value != null) {
      return Image.memory(value);
    } else {
      return Text(
        formattedValue,
        style: widget?.style ?? Theme.of(context).textTheme.subtitle1,
        textAlign:
            widget?.contentAlign ?? CardSettings.of(context).contentAlign,
      );
    }
  }
}
